from typing import Any, TypeVar

from zenoh import ZBytes

_T = TypeVar("_T")

class Int8(int):
    """int subclass enabling to (de)serialize 8bit signed integer."""

class Int16(int):
    """int subclass enabling to (de)serialize 16bit signed integer."""

class Int32(int):
    """int subclass enabling to (de)serialize 32bit signed integer."""

class Int64(int):
    """int subclass enabling to (de)serialize 64bit signed integer."""

class Int128(int):
    """int subclass enabling to (de)serialize 128bit signed integer."""

class UInt8(int):
    """int subclass enabling to (de)serialize 8bit unsigned integer."""

class UInt16(int):
    """int subclass enabling to (de)serialize 16bit unsigned integer."""

class UInt32(int):
    """int subclass enabling to (de)serialize 32bit unsigned integer."""

class UInt64(int):
    """int subclass enabling to (de)serialize 64bit unsigned integer."""

class UInt128(int):
    """int subclass enabling to (de)serialize 128bit unsigned integer."""

class Float32(float):
    """float subclass enabling to (de)serialize 32bit floating point numbers."""

class Float64(float):
    """float subclass enabling to (de)serialize 64bit floating point numbers."""

class ZDeserializeError(Exception):
    pass

def z_serialize(obj: Any) -> ZBytes: ...
def z_deserialize(tp: type[_T], zbytes: ZBytes) -> _T: ...



#
# Copyright (c) 2022 ZettaScale Technology
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors:
#   ZettaScale Zenoh Team, <zenoh@zettascale.tech>
#
import zenoh


def main(conf: zenoh.Config, key: str, timeout: float):
    # initiate logging
    zenoh.init_log_from_env_or("error")

    print("Opening session...")
    with zenoh.open(conf) as session:

        print(f"Sending Liveliness Query '{key}'...")
        replies = session.liveliness().get(key, timeout=timeout)
        for reply in replies:
            try:
                print(f">> Alive token ('{reply.ok.key_expr}')")
            except:
                print(f">> Received (ERROR: '{reply.err.payload.to_string()}')")


# --- Command line argument parsing --- --- --- --- --- ---
if __name__ == "__main__":
    import argparse
    import json

    parser = argparse.ArgumentParser(
        prog="z_get_liveliness", description="zenoh put example"
    )
    parser.add_argument(
        "--mode",
        "-m",
        dest="mode",
        choices=["peer", "client"],
        type=str,
        help="The zenoh session mode.",
    )
    parser.add_argument(
        "--connect",
        "-e",
        dest="connect",
        metavar="ENDPOINT",
        action="append",
        type=str,
        help="Endpoints to connect to.",
    )
    parser.add_argument(
        "--listen",
        "-l",
        dest="listen",
        metavar="ENDPOINT",
        action="append",
        type=str,
        help="Endpoints to listen on.",
    )
    parser.add_argument(
        "--key",
        "-k",
        dest="key",
        default="group1/**",
        type=str,
        help="The key expression to write.",
    )
    parser.add_argument(
        "--timeout",
        "-o",
        dest="timeout",
        default=10.0,
        type=float,
        help="The query timeout.",
    )
    parser.add_argument(
        "--config",
        "-c",
        dest="config",
        metavar="FILE",
        type=str,
        help="A configuration file.",
    )

    args = parser.parse_args()
    conf = (
        zenoh.Config.from_file(args.config)
        if args.config is not None
        else zenoh.Config()
    )
    if args.mode is not None:
        conf.insert_json5("mode", json.dumps(args.mode))
    if args.connect is not None:
        conf.insert_json5("connect/endpoints", json.dumps(args.connect))
    if args.listen is not None:
        conf.insert_json5("listen/endpoints", json.dumps(args.listen))

    main(conf, args.key, args.timeout)



#
# Copyright (c) 2024 ZettaScale Technology
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors:
#   ZettaScale Zenoh Team, <zenoh@zettascale.tech>
#
import ast
import importlib
import inspect
from inspect import Parameter
from pathlib import Path
from typing import Any

PACKAGE = (Path(__file__) / "../../zenoh").resolve()


class CheckExported(ast.NodeVisitor):
    def __init__(self, module: Any):
        self.module = module
        self.current_cls = None

    def visit_ClassDef(self, node: ast.ClassDef):
        # register the current class for method name disambiguation
        self.current_cls = getattr(self.module, node.name)
        getattr(self.current_cls, "__repr__")
        self.generic_visit(node)
        self.current_cls = None

    def visit_FunctionDef(self, node: ast.FunctionDef):
        func = getattr(self.current_cls or self.module, node.name)
        if node.name.startswith("__") or node.name.endswith("serializer"):
            pass
        elif callable(func):
            sig_params = {
                p.name: (p.kind, p.default is not Parameter.empty)
                for p in inspect.signature(func).parameters.values()
                if p.name != "background"
            }
            node_params = {}
            for i, arg in enumerate(node.args.posonlyargs):
                node_params[arg.arg] = (
                    Parameter.POSITIONAL_ONLY,
                    len(node.args.defaults)
                    >= len(node.args.args) + len(node.args.posonlyargs) - i,
                )
            for i, arg in enumerate(node.args.args):
                node_params[arg.arg] = (
                    Parameter.POSITIONAL_OR_KEYWORD,
                    len(node.args.defaults) >= len(node.args.args) - i,
                )
            if arg := node.args.vararg:
                node_params[arg.arg] = (Parameter.VAR_POSITIONAL, False)
            for arg, default in zip(node.args.kwonlyargs, node.args.kw_defaults):
                node_params[arg.arg] = (Parameter.KEYWORD_ONLY, default is not None)
            if arg := node.args.kwarg:
                node_params[arg.arg] = (Parameter.VAR_KEYWORD, False)
            node_params.pop("cls", ...)
            if "self" in node_params:
                node_params["self"] = (Parameter.POSITIONAL_ONLY, False)
            if (param := node_params.get("handler")) and not param[1]:
                return
            assert (
                sig_params == node_params
            ), f"{self.current_cls=}\n{func=}\n{sig_params=}\n{node_params=}"
        else:
            getattr(func, "__get__")

    def visit_AnnAssign(self, node: ast.AnnAssign):
        if self.current_cls is not None:
            assert isinstance(node.target, ast.Name)
            getattr(self.current_cls, node.target.id)


def main():
    for entry in PACKAGE.glob("*.pyi"):
        with open(entry) as f:
            parts = list(entry.relative_to(PACKAGE.parent).parts)
            parts[-1] = parts[-1].rstrip(".pyi")
            module_name = ".".join(p for p in parts if p != "__init__")
            visitor = CheckExported(importlib.import_module(module_name))
            visitor.visit(ast.parse(f.read()))


if __name__ == "__main__":
    main()



#
# Copyright (c) 2024 ZettaScale Technology
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors:
#   ZettaScale Zenoh Team, <zenoh@zettascale.tech>
#
import time

import zenoh


def main(conf: zenoh.Config, payload_size: int, warmup: int, samples: int):
    # initiate logging
    zenoh.init_log_from_env_or("error")

    print("Opening session...")
    with zenoh.open(conf) as session:

        sub = session.declare_subscriber("test/pong")
        pub = session.declare_publisher(
            "test/ping", congestion_control=zenoh.CongestionControl.BLOCK
        )
        data = bytes(i % 10 for i in range(0, payload_size))

        print(f"Warming up for {warmup}...")
        warmup_end = time.time() + warmup
        while time.time() < warmup_end:
            pub.put(data)
            sub.recv()

        sample_list = []
        for i in range(samples):
            write_time = time.time()
            pub.put(data)
            sub.recv()
            sample_list.append(round((time.time() - write_time) * 1_000_000))

        for i, rtt in enumerate(sample_list):
            print(f"{payload_size} bytes: seq={i} rtt={rtt}µs lat={rtt / 2}µs")


# --- Command line argument parsing --- --- --- --- --- ---
if __name__ == "__main__":
    import argparse
    import json

    parser = argparse.ArgumentParser(prog="z_ping", description="zenoh get example")
    parser.add_argument(
        "--mode",
        "-m",
        dest="mode",
        choices=["peer", "client"],
        type=str,
        help="The zenoh session mode.",
    )
    parser.add_argument(
        "--connect",
        "-e",
        dest="connect",
        metavar="ENDPOINT",
        action="append",
        type=str,
        help="Endpoints to connect to.",
    )
    parser.add_argument(
        "--listen",
        "-l",
        dest="listen",
        metavar="ENDPOINT",
        action="append",
        type=str,
        help="Endpoints to listen on.",
    )
    parser.add_argument(
        "--config",
        "-c",
        dest="config",
        metavar="FILE",
        type=str,
        help="A configuration file.",
    )
    parser.add_argument(
        "--warmup",
        "-w",
        dest="warmup",
        metavar="WARMUP",
        type=float,
        default=1.0,
        help="The number of seconds to warmup (float)",
    )
    parser.add_argument(
        "--samples",
        "-n",
        dest="samples",
        metavar="SAMPLES",
        type=int,
        default=100,
        help="The number of round-trip to measure",
    )
    parser.add_argument(
        "payload_size",
        metavar="PAYLOAD_SIZE",
        type=int,
        help="Sets the size of the payload to publish.",
    )

    args = parser.parse_args()
    conf = (
        zenoh.Config.from_file(args.config)
        if args.config is not None
        else zenoh.Config()
    )
    if args.mode is not None:
        conf.insert_json5("mode", json.dumps(args.mode))
    if args.connect is not None:
        conf.insert_json5("connect/endpoints", json.dumps(args.connect))
    if args.listen is not None:
        conf.insert_json5("listen/endpoints", json.dumps(args.listen))

    main(conf, args.payload_size, args.warmup, args.samples)



# Copyright (c) 2017, 2022 ZettaScale Technology Inc.
import sys
import time
from os import getpgid, killpg, path
from signal import SIGINT
from subprocess import PIPE, Popen, TimeoutExpired

import fixtures

# Contributors:
#   ZettaScale Zenoh team, <zenoh@zettascale.tech>
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.

# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0


examples = path.realpath(__file__).split("/tests")[0] + "/examples/"
tab = "\t"
ret = "\r\n"


class Pyrun(fixtures.Fixture):
    def __init__(self, p, args=None) -> None:
        if args is None:
            args = []
        self.name = p
        print(f"starting {self.name}")
        self.process: Popen = Popen(
            ["python3", path.join(examples, p), *args],
            stdout=PIPE,
            stderr=PIPE,
            start_new_session=True,
        )
        self.start = time.time()
        self.end = None
        self.errors = []
        self._stdouts = []
        self._stderrs = []

    def _setUp(self):
        self.addCleanup(self.process.send_signal, SIGINT)

    def dbg(self):
        self.wait()
        print(f"{self.name} stdout:")
        print(f"{tab}{tab.join(self.stdout)}")
        print(f"{self.name} stderr:")
        print(f"{tab}{tab.join(self.stderr)}")

    def status(self, expecting=0):
        status = self.wait()
        formatted = (
            f"{self.name}: returned {status} (expected {-expecting}) - {self.time:.2}s"
        )
        print(formatted)
        return formatted if status != -expecting else None

    def wait(self):
        try:
            code = self.process.wait(timeout=10)
        except TimeoutExpired:
            self.process.send_signal(SIGINT)
            code = self.process.wait(timeout=10)
        if self.end is None:
            self.end = time.time()
        return code

    def interrupt(self):
        # send SIGINT to process group
        pgid = getpgid(self.process.pid)
        killpg(pgid, SIGINT)
        return self.status(SIGINT)

    @property
    def stdout(self):
        self._stdouts.extend(
            line.decode("utf8") for line in self.process.stdout.readlines()
        )
        return self._stdouts

    @property
    def stderr(self):
        self._stderrs.extend(
            line.decode("utf8") for line in self.process.stderr.readlines()
        )
        return self._stderrs

    @property
    def time(self):
        return None if self.end is None else (self.end - self.start)


def test_z_bytes():
    """Test z_bytes."""
    z_bytes = Pyrun("z_bytes.py")
    if sys.version_info >= (3, 9):
        if error := z_bytes.status():
            z_bytes.dbg()
            z_bytes.errors.append(error)

    assert not z_bytes.errors


def test_z_info_z_scout():
    z_info = Pyrun("z_info.py")
    z_scout = Pyrun("z_scout.py")
    if error := z_info.status():
        z_info.dbg()
        z_info.errors.append(error)

    if error := z_scout.status():
        z_scout.dbg()
        z_scout.errors.append(error)

    assert not z_info.errors
    assert not z_scout.errors


def test_z_get_z_queryable():
    """Test z_get & z_queryable"""
    z_queryable = Pyrun("z_queryable.py", ["-k=demo/example/zenoh-python-queryable"])
    time.sleep(3)
    ## z_get: Able to get reply from queryable
    z_get = Pyrun("z_get.py", ["-s=demo/example/zenoh-python-queryable"])
    if error := z_get.status():
        z_get.dbg()
        z_get.errors.append(error)

    z_queryable.interrupt()

    if not (
        "Received ('demo/example/zenoh-python-queryable': 'Queryable from Python!')"
        in "".join(z_get.stdout)
    ):
        z_get.dbg()
        z_queryable.dbg()
        z_get.errors.append("z_get didn't get a response from z_queryable")
    queryableout = "".join(z_queryable.stdout)
    if not ("Received Query 'demo/example/zenoh-python-queryable'" in queryableout):
        z_queryable.errors.append("z_queryable didn't catch query")
    if any(("z_queryable" in error) for error in z_queryable.errors):
        z_queryable.dbg()

    assert not z_get.errors
    assert not z_queryable.errors


def test_z_storage_z_sub():
    """Test z_storage & z_sub."""
    z_storage = Pyrun("z_storage.py")
    z_sub = Pyrun("z_sub.py")
    time.sleep(3)
    ## z_put: Put one message (to storage & sub)
    z_put = Pyrun("z_put.py")
    time.sleep(1)
    ## z_pub: Put two messages (to storage & sub)
    pub = Pyrun("z_pub.py", ["--iter=2"])
    time.sleep(1)
    z_get = Pyrun("z_get.py", ["-s=demo/example/zenoh-python-put"])
    if error := z_put.status():
        z_put.dbg()
        z_put.errors.append(error)

    if error := z_get.status():
        z_get.dbg()
        z_get.errors.append(error)

    if not (
        "Received ('demo/example/zenoh-python-put': 'Put from Python!')"
        in "".join(z_get.stdout)
    ):
        z_get.dbg()
        z_get.errors.append("z_get didn't get a response from z_storage about put")
    if any(("z_get" in error) for error in z_get.errors):
        z_get.dbg()
    time.sleep(1)

    z_delete = Pyrun("z_delete.py")
    if error := z_delete.status():
        z_delete.dbg()
        z_delete.errors.append(error)
    time.sleep(1)

    ## z_get: Unable to get put from storage
    z_get = Pyrun("z_get.py", ["-s=demo/example/zenoh-python-put"])
    if error := z_get.status():
        z_get.dbg()
        z_get.errors.append(error)
    if "Received ('demo/example/zenoh-python-put': 'Put from Python!')" in "".join(
        z_get.stdout
    ):
        z_storage.dbg()
        z_get.errors.append(
            "z_get did get a response from z_storage about put after delete"
        )
    if any(("z_get" in error) for error in z_get.errors):
        z_get.dbg()
    time.sleep(1)

    ## z_sub: Should receive put, pub and delete
    if error := z_sub.process.send_signal(SIGINT):
        z_sub.dbg()
        z_sub.errors.append(error)
    subout = "".join(z_sub.stdout)
    if not (
        "Received SampleKind.PUT ('demo/example/zenoh-python-put': 'Put from Python!')"
        in subout
    ):
        z_sub.errors.append("z_sub didn't catch put")
    if not (
        "Received SampleKind.PUT ('demo/example/zenoh-python-pub': '[   1] Pub from Python!')"
        in subout
    ):
        z_sub.errors.append("z_sub didn't catch second z_pub")
    if not (
        "Received SampleKind.DELETE ('demo/example/zenoh-python-put': '')" in subout
    ):
        z_sub.errors.append("z_sub didn't catch delete")
    if any(("z_sub" in error) for error in z_sub.errors):
        z_sub.dbg()

    ## z_storage: Should receive put, pub, delete, and query
    if error := z_storage.process.send_signal(SIGINT):
        z_storage.dbg()
        z_storage.errors.append(error)
    storageout = "".join(z_storage.stdout)
    if not (
        "Received SampleKind.PUT ('demo/example/zenoh-python-put': 'Put from Python!')"
        in storageout
    ):
        z_storage.errors.append("z_storage didn't catch put")
    if not (
        "Received SampleKind.PUT ('demo/example/zenoh-python-pub': '[   1] Pub from Python!')"
        in storageout
    ):
        z_storage.errors.append("z_storage didn't catch second z_pub")
    if not (
        "Received SampleKind.DELETE ('demo/example/zenoh-python-put': '')" in storageout
    ):
        z_storage.errors.append("z_storage didn't catch delete")
    if not ("Received Query 'demo/example/zenoh-python-put'" in storageout):
        z_storage.errors.append("z_storage didn't catch query")
    if any(("z_storage" in error) for error in z_storage.errors):
        z_storage.dbg()

    assert not z_sub.errors
    assert not z_storage.errors
    assert not z_get.errors


def test_z_pull_z_sub_queued():
    """Test z_pull & z_sub_queued."""
    ## Run z_pull and z_sub_queued
    sub_queued = Pyrun("z_sub_queued.py")
    time.sleep(3)
    pull = Pyrun("z_pull.py", ["--size=1", "--interval=1"])
    time.sleep(3)
    ## z_pub: Put two messages (to storage & sub)
    pub = Pyrun("z_pub.py", ["--iter=2", "--interval=0"])
    if error := pub.status():
        pub.dbg()
        pub.errors.append(error)
    ## z_sub_queued: Should receive two messages
    if error := sub_queued.interrupt():
        sub_queued.dbg()
        sub_queued.errors.append(error)
    sub_queued_out = "".join(sub_queued.stdout)
    if not (
        "Received SampleKind.PUT ('demo/example/zenoh-python-pub': '[   0] Pub from Python!')"
        in sub_queued_out
    ):
        sub_queued.errors.append("z_sub_queued didn't catch the first z_pub")
    if not (
        "Received SampleKind.PUT ('demo/example/zenoh-python-pub': '[   1] Pub from Python!')"
        in sub_queued_out
    ):
        sub_queued.errors.append("z_sub_queued didn't catch the second z_pub")
    if any(("z_sub_queued" in error) for error in sub_queued.errors):
        sub_queued.dbg()
    ## z_pull: Should only receive the last messages
    time.sleep(3)
    if error := pull.interrupt():
        pull.dbg()
        pull.errors.append(error)
    pullout = "".join(pull.stdout)
    if (
        "Received SampleKind.PUT ('demo/example/zenoh-python-pub': '[   0] Pub from Python!')"
        in pullout
    ):
        pull.errors.append("z_pull shouldn't catch the old z_pub")
    if not (
        "Received SampleKind.PUT ('demo/example/zenoh-python-pub': '[   1] Pub from Python!')"
        in pullout
    ):
        pull.errors.append("z_pull didn't catch the last z_pub")
    if any(("z_pull" in error) for error in pull.errors):
        pull.dbg()

    assert not pub.errors
    assert not sub_queued.errors
    assert not pull.errors


def test_z_sub_thr_z_pub_thr():
    """Test z_sub_thr & z_pub_thr."""
    sub_thr = Pyrun("z_sub_thr.py")
    pub_thr = Pyrun("z_pub_thr.py", ["128"])
    time.sleep(5)
    if error := sub_thr.interrupt():
        sub_thr.dbg()
        sub_thr.errors.append(error)
    if error := pub_thr.interrupt():
        pub_thr.dbg()
        pub_thr.errors.append(error)

    assert not sub_thr.errors
    assert not pub_thr.errors



#
# Copyright (c) 2024 ZettaScale Technology
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors:
#   ZettaScale Zenoh Team, <zenoh@zettascale.tech>
#
import sys
from dataclasses import dataclass

import pytest

from zenoh import ZBytes
from zenoh.ext import (
    Float32,
    Float64,
    Int8,
    Int16,
    Int32,
    Int64,
    Int128,
    UInt8,
    UInt16,
    UInt32,
    UInt64,
    UInt128,
    z_deserialize,
    z_serialize,
)

default_serializer_tests = [
    (ZBytes, ZBytes(b"foo")),
    (bytes, b"foo"),
    (bytearray, bytearray(b"foo")),
    (str, "foo"),
    *(
        (tp, tp(i))
        for i in (-42, 42)
        for tp in (int, Int8, Int16, Int32, Int64, Int128)
    ),
    *((tp, tp(42)) for tp in (UInt8, UInt16, UInt32, UInt64, UInt128)),
    (float, 0.5),
    (Float64, Float64(0.5)),
    (Float32, Float32(0.5)),
    (bool, True),
    (list[int], [0, 1, 2]),
    (tuple[int, int], (0, 1)),
    (dict[str, str], {"foo": "bar"}),
    (set[int], {0, 1, 2}),
    (frozenset[int], frozenset([0, 1, 2])),
    (list[tuple[float, Float32]], [(0.0, Float32(0.5)), (1.5, Float32(2))]),
]


@pytest.mark.parametrize("tp, value", default_serializer_tests)
def test_default_serializer(tp, value):
    zbytes = z_serialize(value)
    assert z_deserialize(tp, zbytes) == value



#
# Copyright (c) 2024 ZettaScale Technology
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors:
#   ZettaScale Zenoh Team, <zenoh@zettascale.tech>
#
import time

import zenoh


def main(conf: zenoh.Config, express: bool):
    # initiate logging
    zenoh.init_log_from_env_or("error")

    print("Opening session...")
    with zenoh.open(conf) as session:
        pub = session.declare_publisher(
            "test/pong",
            congestion_control=zenoh.CongestionControl.BLOCK,
            express=express,
        )
        session.declare_subscriber("test/ping", lambda s: pub.put(s.payload))

        print("Press CTRL-C to quit...")
        while True:
            time.sleep(1)


# --- Command line argument parsing --- --- --- --- --- ---
if __name__ == "__main__":
    import argparse
    import json

    parser = argparse.ArgumentParser(prog="z_pong", description="zenoh get example")
    parser.add_argument(
        "--mode",
        "-m",
        dest="mode",
        choices=["peer", "client"],
        type=str,
        help="The zenoh session mode.",
    )
    parser.add_argument(
        "--connect",
        "-e",
        dest="connect",
        metavar="ENDPOINT",
        action="append",
        type=str,
        help="Endpoints to connect to.",
    )
    parser.add_argument(
        "--listen",
        "-l",
        dest="listen",
        metavar="ENDPOINT",
        action="append",
        type=str,
        help="Endpoints to listen on.",
    )
    parser.add_argument(
        "--config",
        "-c",
        dest="config",
        metavar="FILE",
        type=str,
        help="A configuration file.",
    )
    parser.add_argument(
        "--express",
        dest="express",
        metavar="EXPRESS",
        type=bool,
        default=False,
        help="Express publishing",
    )

    args = parser.parse_args()
    conf = (
        zenoh.Config.from_file(args.config)
        if args.config is not None
        else zenoh.Config()
    )
    if args.mode is not None:
        conf.insert_json5("mode", json.dumps(args.mode))
    if args.connect is not None:
        conf.insert_json5("connect/endpoints", json.dumps(args.connect))
    if args.listen is not None:
        conf.insert_json5("listen/endpoints", json.dumps(args.listen))

    main(conf, args.express)



try:
    from zenoh._ext import *
except ImportError:
    import warnings

    raise ModuleNotFoundError(
        "No module named 'zenoh.ext'.\nzenoh must be built wit zenoh-ext feature to enable it."
    ) from None

_INT8_MIN = -(1 << 7)
_INT16_MIN = -(1 << 15)
_INT32_MIN = -(1 << 31)
_INT64_MIN = -(1 << 63)
_INT128_MIN = -(1 << 127)

_INT8_MAX = 1 << 7
_INT16_MAX = 1 << 15
_INT32_MAX = 1 << 31
_INT64_MAX = 1 << 63
_INT128_MAX = 1 << 127

_UINT8_MAX = 1 << 8
_UINT16_MAX = 1 << 16
_UINT32_MAX = 1 << 32
_UINT64_MAX = 1 << 64
_UINT128_MAX = 1 << 128


class Int8(int):
    """int subclass enabling to (de)serialize 8bit signed integer."""

    def __new__(cls, i: int):
        assert _INT8_MIN <= i < _INT8_MAX, f"{i} too big for Int8"
        return int.__new__(cls, i)


class Int16(int):
    """int subclass enabling to (de)serialize 16bit signed integer."""

    def __new__(cls, i: int):
        assert _INT16_MIN <= i < _INT16_MAX, f"{i} too big for Int16"
        return int.__new__(cls, i)


class Int32(int):
    """int subclass enabling to (de)serialize 32bit signed integer."""

    def __new__(cls, i: int):
        assert _INT32_MIN <= i < _INT32_MAX, f"{i} too big for Int32"
        return int.__new__(cls, i)


class Int64(int):
    """int subclass enabling to (de)serialize 64bit signed integer."""

    def __new__(cls, i: int):
        assert _INT64_MIN <= i < _INT64_MAX, f"{i} too big for Int64"
        return int.__new__(cls, i)


class Int128(int):
    """int subclass enabling to (de)serialize 128bit signed integer."""

    def __new__(cls, i: int):
        assert _INT128_MIN <= i < _INT128_MAX, f"{i} too big for Int128"
        return int.__new__(cls, i)


class UInt8(int):
    """int subclass enabling to (de)serialize 8bit unsigned integer."""

    def __new__(cls, i: int):
        assert 0 <= i < _UINT8_MAX, f"{i} too big for UInt8"
        return int.__new__(cls, i)


class UInt16(int):
    """int subclass enabling to (de)serialize 16bit unsigned integer."""

    def __new__(cls, i: int):
        assert 0 <= i < _UINT16_MAX, f"{i} too big for UInt16"
        return int.__new__(cls, i)


class UInt32(int):
    """int subclass enabling to (de)serialize 32bit unsigned integer."""

    def __new__(cls, i: int):
        assert 0 <= i < _UINT32_MAX, f"{i} too big for UInt32"
        return int.__new__(cls, i)


class UInt64(int):
    """int subclass enabling to (de)serialize 64bit unsigned integer."""

    def __new__(cls, i: int):
        assert 0 <= i < _UINT64_MAX, f"{i} too big for UInt64"
        return int.__new__(cls, i)


class UInt128(int):
    """int subclass enabling to (de)serialize 128bit unsigned integer."""

    def __new__(cls, i: int):
        assert 0 <= i < _UINT128_MAX, f"{i} too big for UInt128"
        return int.__new__(cls, i)


class Float32(float):
    """float subclass enabling to (de)serialize 32bit floating point numbers."""


class Float64(float):
    """float subclass enabling to (de)serialize 64bit floating point numbers."""
